@@ -14,7 +14,7 @@ if "--fail" in sys.argv:
 
 setup(
     name="chattymodule",
-    version='0.0.1',
+    version="0.0.1",
     description="A sample Python project with a single module",
-    py_modules=['chattymodule'],
+    py_modules=["chattymodule"],
 )
