@@ -16,7 +16,7 @@ from tests.lib.local_repos import local_checkout
 from tests.lib.path import Path
 
 
-class ArgRecordingSdist(object):
+class ArgRecordingSdist:
     def __init__(self, sdist_path, args_path):
         self.sdist_path = sdist_path
         self._args_path = args_path
@@ -43,46 +43,41 @@ def arg_recording_sdist_maker(script):
         setup(name={name!r}, version="0.1.0")
         """
     )
-    output_dir = script.scratch_path.joinpath(
-        "args_recording_sdist_maker_output"
-    )
+    output_dir = script.scratch_path.joinpath("args_recording_sdist_maker_output")
     output_dir.mkdir(parents=True)
     script.environ["OUTPUT_DIR"] = str(output_dir)
 
-    def _arg_recording_sdist_maker(name):
-        # type: (str) -> ArgRecordingSdist
+    def _arg_recording_sdist_maker(name: str) -> ArgRecordingSdist:
         extra_files = {"setup.py": arg_writing_setup_py.format(name=name)}
-        sdist_path = create_basic_sdist_for_package(
-            script, name, "0.1.0", extra_files
-        )
-        args_path = output_dir / "{}.json".format(name)
+        sdist_path = create_basic_sdist_for_package(script, name, "0.1.0", extra_files)
+        args_path = output_dir / f"{name}.json"
         return ArgRecordingSdist(sdist_path, args_path)
 
     return _arg_recording_sdist_maker
 
 
 @pytest.mark.network
-def test_requirements_file(script, with_wheel):
+@pytest.mark.usefixtures("with_wheel")
+def test_requirements_file(script):
     """
     Test installing from a requirements file.
 
     """
-    other_lib_name, other_lib_version = 'anyjson', '0.3'
-    script.scratch_path.joinpath("initools-req.txt").write_text(textwrap.dedent("""\
+    other_lib_name, other_lib_version = "peppercorn", "0.6"
+    script.scratch_path.joinpath("initools-req.txt").write_text(
+        textwrap.dedent(
+            f"""\
         INITools==0.2
         # and something else to test out:
         {other_lib_name}<={other_lib_version}
-        """.format(**locals())))
-    result = script.pip(
-        'install', '-r', script.scratch_path / 'initools-req.txt'
+        """
+        )
     )
-    result.did_create(
-        script.site_packages / 'INITools-0.2.dist-info'
-    )
-    result.did_create(script.site_packages / 'initools')
+    result = script.pip("install", "-r", script.scratch_path / "initools-req.txt")
+    result.did_create(script.site_packages / "INITools-0.2.dist-info")
+    result.did_create(script.site_packages / "initools")
     assert result.files_created[script.site_packages / other_lib_name].dir
-    fn = '{}-{}.dist-info'.format(
-        other_lib_name, other_lib_version)
+    fn = "{}-{}.dist-info".format(other_lib_name, other_lib_version)
     assert result.files_created[script.site_packages / fn].dir
 
 
@@ -99,41 +94,36 @@ def test_schema_check_in_requirements_file(script):
     )
 
     with pytest.raises(AssertionError):
-        script.pip(
-            "install", "-vvv", "-r", script.scratch_path / "file-egg-req.txt"
-        )
+        script.pip("install", "-vvv", "-r", script.scratch_path / "file-egg-req.txt")
 
 
-@pytest.mark.parametrize("test_type,editable", [
-    ("rel_path", False),
-    ("rel_path", True),
-    ("rel_url", False),
-    ("rel_url", True),
-    ("embedded_rel_path", False),
-    ("embedded_rel_path", True),
-])
-def test_relative_requirements_file(
-    script, data, test_type, editable, with_wheel
-):
+@pytest.mark.parametrize(
+    "test_type,editable",
+    [
+        ("rel_path", False),
+        ("rel_path", True),
+        ("rel_url", False),
+        ("rel_url", True),
+        ("embedded_rel_path", False),
+        ("embedded_rel_path", True),
+    ],
+)
+@pytest.mark.usefixtures("with_wheel")
+def test_relative_requirements_file(script, data, test_type, editable):
     """
     Test installing from a requirements file with a relative path. For path
     URLs, use an egg= definition.
 
     """
-    dist_info_folder = (
-        script.site_packages /
-        'FSPkg-0.1.dev0.dist-info'
-    )
-    egg_link_file = (
-        script.site_packages / 'FSPkg.egg-link'
-    )
-    package_folder = script.site_packages / 'fspkg'
+    dist_info_folder = script.site_packages / "FSPkg-0.1.dev0.dist-info"
+    egg_link_file = script.site_packages / "FSPkg.egg-link"
+    package_folder = script.site_packages / "fspkg"
 
     # Compute relative install path to FSPkg from scratch path.
     full_rel_path = Path(
-        os.path.relpath(data.packages.joinpath('FSPkg'), script.scratch_path)
+        os.path.relpath(data.packages.joinpath("FSPkg"), script.scratch_path)
     )
-    full_rel_url = 'file:' + full_rel_path + '#egg=FSPkg'
+    full_rel_url = "file:" + full_rel_path + "#egg=FSPkg"
     embedded_rel_path = script.scratch_path.joinpath(full_rel_path)
 
     req_path = {
@@ -142,170 +132,190 @@ def test_relative_requirements_file(
         "embedded_rel_path": embedded_rel_path,
     }[test_type]
 
-    req_path = req_path.replace(os.path.sep, '/')
+    req_path = req_path.replace(os.path.sep, "/")
     # Install as either editable or not.
     if not editable:
-        with requirements_file(req_path + '\n',
-                               script.scratch_path) as reqs_file:
-            result = script.pip('install', '-vvv', '-r', reqs_file.name,
-                                cwd=script.scratch_path)
+        with requirements_file(req_path + "\n", script.scratch_path) as reqs_file:
+            result = script.pip(
+                "install", "-vvv", "-r", reqs_file.name, cwd=script.scratch_path
+            )
             result.did_create(dist_info_folder)
             result.did_create(package_folder)
     else:
-        with requirements_file('-e ' + req_path + '\n',
-                               script.scratch_path) as reqs_file:
-            result = script.pip('install', '-vvv', '-r', reqs_file.name,
-                                cwd=script.scratch_path)
+        with requirements_file(
+            "-e " + req_path + "\n", script.scratch_path
+        ) as reqs_file:
+            result = script.pip(
+                "install", "-vvv", "-r", reqs_file.name, cwd=script.scratch_path
+            )
             result.did_create(egg_link_file)
 
 
 @pytest.mark.xfail
 @pytest.mark.network
 @need_svn
-def test_multiple_requirements_files(script, tmpdir, with_wheel):
+@pytest.mark.usefixtures("with_wheel")
+def test_multiple_requirements_files(script, tmpdir):
     """
     Test installing from multiple nested requirements files.
 
     """
-    other_lib_name, other_lib_version = 'anyjson', '0.3'
+    other_lib_name, other_lib_version = "six", "1.16.0"
     script.scratch_path.joinpath("initools-req.txt").write_text(
-        textwrap.dedent("""
+        textwrap.dedent(
+            """
             -e {}@10#egg=INITools
             -r {}-req.txt
-        """).format
-        (
-            local_checkout('svn+http://svn.colorstudy.com/INITools', tmpdir),
-            other_lib_name
+        """
+        ).format(
+            local_checkout("svn+http://svn.colorstudy.com/INITools", tmpdir),
+            other_lib_name,
         ),
     )
-    script.scratch_path.joinpath(
-        "{other_lib_name}-req.txt".format(**locals())).write_text(
-            "{other_lib_name}<={other_lib_version}".format(**locals())
-    )
-    result = script.pip(
-        'install', '-r', script.scratch_path / 'initools-req.txt'
+    script.scratch_path.joinpath(f"{other_lib_name}-req.txt").write_text(
+        f"{other_lib_name}<={other_lib_version}"
     )
+    result = script.pip("install", "-r", script.scratch_path / "initools-req.txt")
     assert result.files_created[script.site_packages / other_lib_name].dir
-    fn = '{other_lib_name}-{other_lib_version}.dist-info'.format(**locals())
+    fn = f"{other_lib_name}-{other_lib_version}.dist-info"
     assert result.files_created[script.site_packages / fn].dir
-    result.did_create(script.venv / 'src' / 'initools')
+    result.did_create(script.venv / "src" / "initools")
 
 
 def test_package_in_constraints_and_dependencies(script, data):
     script.scratch_path.joinpath("constraints.txt").write_text(
         "TopoRequires2==0.0.1\nTopoRequires==0.0.1"
     )
-    result = script.pip('install', '--no-index', '-f',
-                        data.find_links, '-c', script.scratch_path /
-                        'constraints.txt', 'TopoRequires2')
-    assert 'installed TopoRequires-0.0.1' in result.stdout
+    result = script.pip(
+        "install",
+        "--no-index",
+        "-f",
+        data.find_links,
+        "-c",
+        script.scratch_path / "constraints.txt",
+        "TopoRequires2",
+    )
+    assert "installed TopoRequires-0.0.1" in result.stdout
 
 
 def test_multiple_constraints_files(script, data):
     script.scratch_path.joinpath("outer.txt").write_text("-c inner.txt")
-    script.scratch_path.joinpath("inner.txt").write_text(
-        "Upper==1.0")
+    script.scratch_path.joinpath("inner.txt").write_text("Upper==1.0")
     result = script.pip(
-        'install', '--no-index', '-f', data.find_links, '-c',
-        script.scratch_path / 'outer.txt', 'Upper')
-    assert 'installed Upper-1.0' in result.stdout
+        "install",
+        "--no-index",
+        "-f",
+        data.find_links,
+        "-c",
+        script.scratch_path / "outer.txt",
+        "Upper",
+    )
+    assert "installed Upper-1.0" in result.stdout
 
 
-@pytest.mark.xfail(reason="Unclear what this guarantee is for.")
+# FIXME: Unclear what this guarantee is for.
 def test_respect_order_in_requirements_file(script, data):
-    script.scratch_path.joinpath("frameworks-req.txt").write_text(textwrap.dedent("""\
+    script.scratch_path.joinpath("frameworks-req.txt").write_text(
+        textwrap.dedent(
+            """\
         parent
         child
         simple
-        """))
+        """
+        )
+    )
 
     result = script.pip(
-        'install', '--no-index', '-f', data.find_links, '-r',
-        script.scratch_path / 'frameworks-req.txt'
+        "install",
+        "--no-index",
+        "-f",
+        data.find_links,
+        "-r",
+        script.scratch_path / "frameworks-req.txt",
     )
 
-    downloaded = [line for line in result.stdout.split('\n')
-                  if 'Processing' in line]
+    downloaded = [line for line in result.stdout.split("\n") if "Processing" in line]
 
-    assert 'parent' in downloaded[0], (
-        'First download should be "parent" but was "{}"'.format(downloaded[0])
-    )
-    assert 'child' in downloaded[1], (
-        'Second download should be "child" but was "{}"'.format(downloaded[1])
-    )
-    assert 'simple' in downloaded[2], (
-        'Third download should be "simple" but was "{}"'.format(downloaded[2])
-    )
+    assert (
+        "parent" in downloaded[0]
+    ), 'First download should be "parent" but was "{}"'.format(downloaded[0])
+    assert (
+        "child" in downloaded[1]
+    ), 'Second download should be "child" but was "{}"'.format(downloaded[1])
+    assert (
+        "simple" in downloaded[2]
+    ), 'Third download should be "simple" but was "{}"'.format(downloaded[2])
 
 
 def test_install_local_editable_with_extras(script, data):
     to_install = data.packages.joinpath("LocalExtras")
     res = script.pip_install_local(
-        '-e', to_install + '[bar]', allow_stderr_warning=True
+        "-e", to_install + "[bar]", allow_stderr_warning=True
     )
-    res.did_update(script.site_packages / 'easy-install.pth')
-    res.did_create(script.site_packages / 'LocalExtras.egg-link')
-    res.did_create(script.site_packages / 'simple')
+    res.did_update(script.site_packages / "easy-install.pth")
+    res.did_create(script.site_packages / "LocalExtras.egg-link")
+    res.did_create(script.site_packages / "simple")
 
 
 def test_install_collected_dependencies_first(script):
     result = script.pip_install_local(
-        'toporequires2',
+        "toporequires2",
     )
-    text = [line for line in result.stdout.split('\n')
-            if 'Installing' in line][0]
-    assert text.endswith('toporequires2')
+    text = [line for line in result.stdout.split("\n") if "Installing" in line][0]
+    assert text.endswith("toporequires2")
 
 
 @pytest.mark.network
 def test_install_local_editable_with_subdirectory(script):
-    version_pkg_path = _create_test_package_with_subdirectory(script,
-                                                              'version_subdir')
+    version_pkg_path = _create_test_package_with_subdirectory(script, "version_subdir")
     result = script.pip(
-        'install', '-e',
-        '{uri}#egg=version_subpkg&subdirectory=version_subdir'.format(
-            uri='git+' + path_to_url(version_pkg_path),
+        "install",
+        "-e",
+        "{uri}#egg=version_subpkg&subdirectory=version_subdir".format(
+            uri="git+" + path_to_url(version_pkg_path),
         ),
     )
 
-    result.assert_installed('version-subpkg', sub_dir='version_subdir')
+    result.assert_installed("version-subpkg", sub_dir="version_subdir")
 
 
 @pytest.mark.network
 def test_install_local_with_subdirectory(script):
-    version_pkg_path = _create_test_package_with_subdirectory(script,
-                                                              'version_subdir')
+    version_pkg_path = _create_test_package_with_subdirectory(script, "version_subdir")
     result = script.pip(
-        'install',
-        '{uri}#egg=version_subpkg&subdirectory=version_subdir'.format(
-            uri='git+' + path_to_url(version_pkg_path),
+        "install",
+        "{uri}#egg=version_subpkg&subdirectory=version_subdir".format(
+            uri="git+" + path_to_url(version_pkg_path),
         ),
     )
 
-    result.assert_installed('version_subpkg.py', editable=False)
+    result.assert_installed("version_subpkg.py", editable=False)
 
 
 @pytest.mark.incompatible_with_test_venv
-def test_wheel_user_with_prefix_in_pydistutils_cfg(
-        script, data, with_wheel):
-    if os.name == 'posix':
+@pytest.mark.usefixtures("with_wheel")
+def test_wheel_user_with_prefix_in_pydistutils_cfg(script, data):
+    if os.name == "posix":
         user_filename = ".pydistutils.cfg"
     else:
         user_filename = "pydistutils.cfg"
-    user_cfg = os.path.join(os.path.expanduser('~'), user_filename)
+    user_cfg = os.path.join(os.path.expanduser("~"), user_filename)
     script.scratch_path.joinpath("bin").mkdir()
     with open(user_cfg, "w") as cfg:
-        cfg.write(textwrap.dedent("""
+        cfg.write(
+            textwrap.dedent(
+                f"""
             [install]
-            prefix={script.scratch_path}""".format(**locals())))
+            prefix={script.scratch_path}"""
+            )
+        )
 
     result = script.pip(
-        'install', '--user', '--no-index',
-        '-f', data.find_links,
-        'requiresupper')
+        "install", "--user", "--no-index", "-f", data.find_links, "requiresupper"
+    )
     # Check that we are really installing a wheel
-    assert 'Running setup.py install for requiresupper' not in result.stdout
-    assert 'installed requiresupper' in result.stdout
+    assert "Running setup.py install for requiresupper" not in result.stdout
+    assert "installed requiresupper" in result.stdout
 
 
 def test_install_option_in_requirements_file_overrides_cli(
@@ -317,28 +327,45 @@ def test_install_option_in_requirements_file_overrides_cli(
     reqs_file.write_text("simple --install-option='-O0'")
 
     script.pip(
-        'install', '--no-index', '-f', str(simple_sdist.sdist_path.parent),
-        '-r', str(reqs_file), '--install-option=-O1',
+        "install",
+        "--no-index",
+        "-f",
+        str(simple_sdist.sdist_path.parent),
+        "-r",
+        str(reqs_file),
+        "--install-option=-O1",
     )
     simple_args = simple_sdist.args()
-    assert 'install' in simple_args
-    assert simple_args.index('-O1') < simple_args.index('-O0')
+    assert "install" in simple_args
+    assert simple_args.index("-O1") < simple_args.index("-O0")
 
 
 def test_constraints_not_installed_by_default(script, data):
     script.scratch_path.joinpath("c.txt").write_text("requiresupper")
     result = script.pip(
-        'install', '--no-index', '-f', data.find_links, '-c',
-        script.scratch_path / 'c.txt', 'Upper')
-    assert 'requiresupper' not in result.stdout
+        "install",
+        "--no-index",
+        "-f",
+        data.find_links,
+        "-c",
+        script.scratch_path / "c.txt",
+        "Upper",
+    )
+    assert "requiresupper" not in result.stdout
 
 
 def test_constraints_only_causes_error(script, data):
     script.scratch_path.joinpath("c.txt").write_text("requiresupper")
     result = script.pip(
-        'install', '--no-index', '-f', data.find_links, '-c',
-        script.scratch_path / 'c.txt', expect_error=True)
-    assert 'installed requiresupper' not in result.stdout
+        "install",
+        "--no-index",
+        "-f",
+        data.find_links,
+        "-c",
+        script.scratch_path / "c.txt",
+        expect_error=True,
+    )
+    assert "installed requiresupper" not in result.stdout
 
 
 def test_constraints_local_editable_install_causes_error(
@@ -346,28 +373,32 @@ def test_constraints_local_editable_install_causes_error(
     data,
     resolver_variant,
 ):
-    script.scratch_path.joinpath("constraints.txt").write_text(
-        "singlemodule==0.0.0"
-    )
+    script.scratch_path.joinpath("constraints.txt").write_text("singlemodule==0.0.0")
     to_install = data.src.joinpath("singlemodule")
     result = script.pip(
-        'install', '--no-index', '-f', data.find_links, '-c',
-        script.scratch_path / 'constraints.txt', '-e',
-        to_install, expect_error=True)
+        "install",
+        "--no-index",
+        "-f",
+        data.find_links,
+        "-c",
+        script.scratch_path / "constraints.txt",
+        "-e",
+        to_install,
+        expect_error=True,
+    )
     if resolver_variant == "legacy-resolver":
-        assert 'Could not satisfy constraints' in result.stderr, str(result)
+        assert "Could not satisfy constraints" in result.stderr, str(result)
     else:
         # Because singlemodule only has 0.0.1 available.
-        assert 'No matching distribution found' in result.stderr, str(result)
+        assert "Cannot install singlemodule 0.0.1" in result.stderr, str(result)
 
 
 @pytest.mark.network
 def test_constraints_local_editable_install_pep518(script, data):
     to_install = data.src.joinpath("pep518-3.0")
 
-    script.pip('download', 'setuptools', 'wheel', '-d', data.packages)
-    script.pip(
-        'install', '--no-index', '-f', data.find_links, '-e', to_install)
+    script.pip("download", "setuptools", "wheel", "-d", data.packages)
+    script.pip("install", "--no-index", "-f", data.find_links, "-e", to_install)
 
 
 def test_constraints_local_install_causes_error(
@@ -375,19 +406,23 @@ def test_constraints_local_install_causes_error(
     data,
     resolver_variant,
 ):
-    script.scratch_path.joinpath("constraints.txt").write_text(
-        "singlemodule==0.0.0"
-    )
+    script.scratch_path.joinpath("constraints.txt").write_text("singlemodule==0.0.0")
     to_install = data.src.joinpath("singlemodule")
     result = script.pip(
-        'install', '--no-index', '-f', data.find_links, '-c',
-        script.scratch_path / 'constraints.txt',
-        to_install, expect_error=True)
+        "install",
+        "--no-index",
+        "-f",
+        data.find_links,
+        "-c",
+        script.scratch_path / "constraints.txt",
+        to_install,
+        expect_error=True,
+    )
     if resolver_variant == "legacy-resolver":
-        assert 'Could not satisfy constraints' in result.stderr, str(result)
+        assert "Could not satisfy constraints" in result.stderr, str(result)
     else:
         # Because singlemodule only has 0.0.1 available.
-        assert 'No matching distribution found' in result.stderr, str(result)
+        assert "Cannot install singlemodule 0.0.1" in result.stderr, str(result)
 
 
 def test_constraints_constrain_to_local_editable(
@@ -400,15 +435,20 @@ def test_constraints_constrain_to_local_editable(
         "-e {url}#egg=singlemodule".format(url=path_to_url(to_install))
     )
     result = script.pip(
-        'install', '--no-index', '-f', data.find_links, '-c',
-        script.scratch_path / 'constraints.txt', 'singlemodule',
+        "install",
+        "--no-index",
+        "-f",
+        data.find_links,
+        "-c",
+        script.scratch_path / "constraints.txt",
+        "singlemodule",
         allow_stderr_warning=True,
         expect_error=(resolver_variant == "2020-resolver"),
     )
     if resolver_variant == "2020-resolver":
-        assert 'Links are not allowed as constraints' in result.stderr
+        assert "Editable requirements are not allowed as constraints" in result.stderr
     else:
-        assert 'Running setup.py develop for singlemodule' in result.stdout
+        assert "Running setup.py develop for singlemodule" in result.stdout
 
 
 def test_constraints_constrain_to_local(script, data, resolver_variant):
@@ -417,36 +457,37 @@ def test_constraints_constrain_to_local(script, data, resolver_variant):
         "{url}#egg=singlemodule".format(url=path_to_url(to_install))
     )
     result = script.pip(
-        'install', '--no-index', '-f', data.find_links, '-c',
-        script.scratch_path / 'constraints.txt', 'singlemodule',
+        "install",
+        "--no-index",
+        "-f",
+        data.find_links,
+        "-c",
+        script.scratch_path / "constraints.txt",
+        "singlemodule",
         allow_stderr_warning=True,
-        expect_error=(resolver_variant == "2020-resolver"),
     )
-    if resolver_variant == "2020-resolver":
-        assert 'Links are not allowed as constraints' in result.stderr
-    else:
-        assert 'Running setup.py install for singlemodule' in result.stdout
+    assert "Running setup.py install for singlemodule" in result.stdout
 
 
-def test_constrained_to_url_install_same_url(script, data, resolver_variant):
+def test_constrained_to_url_install_same_url(script, data):
     to_install = data.src.joinpath("singlemodule")
     constraints = path_to_url(to_install) + "#egg=singlemodule"
     script.scratch_path.joinpath("constraints.txt").write_text(constraints)
     result = script.pip(
-        'install', '--no-index', '-f', data.find_links, '-c',
-        script.scratch_path / 'constraints.txt', to_install,
+        "install",
+        "--no-index",
+        "-f",
+        data.find_links,
+        "-c",
+        script.scratch_path / "constraints.txt",
+        to_install,
         allow_stderr_warning=True,
-        expect_error=(resolver_variant == "2020-resolver"),
     )
-    if resolver_variant == "2020-resolver":
-        assert 'Links are not allowed as constraints' in result.stderr
-    else:
-        assert ('Running setup.py install for singlemodule'
-                in result.stdout), str(result)
+    assert "Running setup.py install for singlemodule" in result.stdout, str(result)
 
 
-def test_double_install_spurious_hash_mismatch(
-        script, tmpdir, data, with_wheel):
+@pytest.mark.usefixtures("with_wheel")
+def test_double_install_spurious_hash_mismatch(script, tmpdir, data):
     """Make sure installing the same hashed sdist twice doesn't throw hash
     mismatch errors.
 
@@ -457,26 +498,32 @@ def test_double_install_spurious_hash_mismatch(
 
     """
     # Install wheel package, otherwise, it won't try to build wheels.
-    with requirements_file('simple==1.0 --hash=sha256:393043e672415891885c9a2a'
-                           '0929b1af95fb866d6ca016b42d2e6ce53619b653',
-                           tmpdir) as reqs_file:
+    with requirements_file(
+        "simple==1.0 --hash=sha256:393043e672415891885c9a2a"
+        "0929b1af95fb866d6ca016b42d2e6ce53619b653",
+        tmpdir,
+    ) as reqs_file:
         # Install a package (and build its wheel):
         result = script.pip_install_local(
-            '--find-links', data.find_links,
-            '-r', reqs_file.resolve(),
+            "--find-links",
+            data.find_links,
+            "-r",
+            reqs_file.resolve(),
         )
-        assert 'Successfully installed simple-1.0' in str(result)
+        assert "Successfully installed simple-1.0" in str(result)
 
         # Uninstall it:
-        script.pip('uninstall', '-y', 'simple')
+        script.pip("uninstall", "-y", "simple")
 
         # Then install it again. We should not hit a hash mismatch, and the
         # package should install happily.
         result = script.pip_install_local(
-            '--find-links', data.find_links,
-            '-r', reqs_file.resolve(),
+            "--find-links",
+            data.find_links,
+            "-r",
+            reqs_file.resolve(),
         )
-        assert 'Successfully installed simple-1.0' in str(result)
+        assert "Successfully installed simple-1.0" in str(result)
 
 
 def test_install_with_extras_from_constraints(script, data, resolver_variant):
@@ -485,14 +532,16 @@ def test_install_with_extras_from_constraints(script, data, resolver_variant):
         "{url}#egg=LocalExtras[bar]".format(url=path_to_url(to_install))
     )
     result = script.pip_install_local(
-        '-c', script.scratch_path / 'constraints.txt', 'LocalExtras',
+        "-c",
+        script.scratch_path / "constraints.txt",
+        "LocalExtras",
         allow_stderr_warning=True,
         expect_error=(resolver_variant == "2020-resolver"),
     )
     if resolver_variant == "2020-resolver":
-        assert 'Links are not allowed as constraints' in result.stderr
+        assert "Constraints cannot have extras" in result.stderr
     else:
-        result.did_create(script.site_packages / 'simple')
+        result.did_create(script.site_packages / "simple")
 
 
 def test_install_with_extras_from_install(script):
@@ -504,11 +553,13 @@ def test_install_with_extras_from_install(script):
     )
     script.scratch_path.joinpath("constraints.txt").write_text("LocalExtras")
     result = script.pip_install_local(
-        '--find-links', script.scratch_path,
-        '-c', script.scratch_path / 'constraints.txt',
-        'LocalExtras[baz]',
+        "--find-links",
+        script.scratch_path,
+        "-c",
+        script.scratch_path / "constraints.txt",
+        "LocalExtras[baz]",
     )
-    result.did_create(script.site_packages / 'singlemodule.py')
+    result.did_create(script.site_packages / "singlemodule.py")
 
 
 def test_install_with_extras_joined(script, data, resolver_variant):
@@ -517,15 +568,17 @@ def test_install_with_extras_joined(script, data, resolver_variant):
         "{url}#egg=LocalExtras[bar]".format(url=path_to_url(to_install))
     )
     result = script.pip_install_local(
-        '-c', script.scratch_path / 'constraints.txt', 'LocalExtras[baz]',
+        "-c",
+        script.scratch_path / "constraints.txt",
+        "LocalExtras[baz]",
         allow_stderr_warning=True,
         expect_error=(resolver_variant == "2020-resolver"),
     )
     if resolver_variant == "2020-resolver":
-        assert 'Links are not allowed as constraints' in result.stderr
+        assert "Constraints cannot have extras" in result.stderr
     else:
-        result.did_create(script.site_packages / 'simple')
-        result.did_create(script.site_packages / 'singlemodule.py')
+        result.did_create(script.site_packages / "simple")
+        result.did_create(script.site_packages / "singlemodule.py")
 
 
 def test_install_with_extras_editable_joined(script, data, resolver_variant):
@@ -534,24 +587,27 @@ def test_install_with_extras_editable_joined(script, data, resolver_variant):
         "-e {url}#egg=LocalExtras[bar]".format(url=path_to_url(to_install))
     )
     result = script.pip_install_local(
-        '-c', script.scratch_path / 'constraints.txt', 'LocalExtras[baz]',
+        "-c",
+        script.scratch_path / "constraints.txt",
+        "LocalExtras[baz]",
         allow_stderr_warning=True,
         expect_error=(resolver_variant == "2020-resolver"),
     )
     if resolver_variant == "2020-resolver":
-        assert 'Links are not allowed as constraints' in result.stderr
+        assert "Editable requirements are not allowed as constraints" in result.stderr
     else:
-        result.did_create(script.site_packages / 'simple')
-        result.did_create(script.site_packages / 'singlemodule.py')
+        result.did_create(script.site_packages / "simple")
+        result.did_create(script.site_packages / "singlemodule.py")
 
 
 def test_install_distribution_full_union(script, data):
     to_install = data.packages.joinpath("LocalExtras")
     result = script.pip_install_local(
-        to_install, to_install + "[bar]", to_install + "[baz]")
-    assert 'Running setup.py install for LocalExtras' in result.stdout
-    result.did_create(script.site_packages / 'simple')
-    result.did_create(script.site_packages / 'singlemodule.py')
+        to_install, to_install + "[bar]", to_install + "[baz]"
+    )
+    assert "Running setup.py install for LocalExtras" in result.stdout
+    result.did_create(script.site_packages / "simple")
+    result.did_create(script.site_packages / "singlemodule.py")
 
 
 def test_install_distribution_duplicate_extras(script, data):
@@ -559,8 +615,7 @@ def test_install_distribution_duplicate_extras(script, data):
     package_name = to_install + "[bar]"
     with pytest.raises(AssertionError):
         result = script.pip_install_local(package_name, package_name)
-        expected = (
-            'Double requirement given: {package_name}'.format(**locals()))
+        expected = f"Double requirement given: {package_name}"
         assert expected in result.stderr
 
 
@@ -570,19 +625,20 @@ def test_install_distribution_union_with_constraints(
     resolver_variant,
 ):
     to_install = data.packages.joinpath("LocalExtras")
-    script.scratch_path.joinpath("constraints.txt").write_text(
-        "{to_install}[bar]".format(**locals()))
+    script.scratch_path.joinpath("constraints.txt").write_text(f"{to_install}[bar]")
     result = script.pip_install_local(
-        '-c', script.scratch_path / 'constraints.txt', to_install + '[baz]',
+        "-c",
+        script.scratch_path / "constraints.txt",
+        to_install + "[baz]",
         allow_stderr_warning=True,
         expect_error=(resolver_variant == "2020-resolver"),
     )
     if resolver_variant == "2020-resolver":
-        msg = 'Unnamed requirements are not allowed as constraints'
+        msg = "Unnamed requirements are not allowed as constraints"
         assert msg in result.stderr
     else:
-        assert 'Running setup.py install for LocalExtras' in result.stdout
-        result.did_create(script.site_packages / 'singlemodule.py')
+        assert "Running setup.py install for LocalExtras" in result.stdout
+        result.did_create(script.site_packages / "singlemodule.py")
 
 
 def test_install_distribution_union_with_versions(
@@ -599,16 +655,11 @@ def test_install_distribution_union_with_versions(
     )
     if resolver_variant == "2020-resolver":
         assert "Cannot install localextras[bar]" in result.stderr
-        assert (
-            "localextras[bar] 0.0.1 depends on localextras 0.0.1"
-        ) in result.stdout
-        assert (
-            "localextras[baz] 0.0.2 depends on localextras 0.0.2"
-        ) in result.stdout
+        assert ("localextras[bar] 0.0.1 depends on localextras 0.0.1") in result.stdout
+        assert ("localextras[baz] 0.0.2 depends on localextras 0.0.2") in result.stdout
     else:
         assert (
-            "Successfully installed LocalExtras-0.0.1 simple-3.0 "
-            "singlemodule-0.0.1"
+            "Successfully installed LocalExtras-0.0.1 simple-3.0 singlemodule-0.0.1"
         ) in result.stdout
 
 
@@ -619,27 +670,30 @@ def test_install_distribution_union_conflicting_extras(script, data):
     # and simple==2.0. Once a resolver is added, this conflict should be
     # detected.
     to_install = data.packages.joinpath("LocalExtras-0.0.2")
-    result = script.pip_install_local(to_install, to_install + "[bar]",
-                                      expect_error=True)
-    assert 'installed' not in result.stdout
+    result = script.pip_install_local(
+        to_install, to_install + "[bar]", expect_error=True
+    )
+    assert "installed" not in result.stdout
     assert "Conflict" in result.stderr
 
 
 def test_install_unsupported_wheel_link_with_marker(script):
     script.scratch_path.joinpath("with-marker.txt").write_text(
-        textwrap.dedent("""\
+        textwrap.dedent(
+            """\
             {url}; {req}
-        """).format(
-            url='https://github.com/a/b/c/asdf-1.5.2-cp27-none-xyz.whl',
+        """
+        ).format(
+            url="https://github.com/a/b/c/asdf-1.5.2-cp27-none-xyz.whl",
             req='sys_platform == "xyz"',
         )
     )
-    result = script.pip(
-        'install', '-r', script.scratch_path / 'with-marker.txt'
-    )
+    result = script.pip("install", "-r", script.scratch_path / "with-marker.txt")
 
-    assert ("Ignoring asdf: markers 'sys_platform == \"xyz\"' don't match "
-            "your environment") in result.stdout
+    assert (
+        "Ignoring asdf: markers 'sys_platform == \"xyz\"' don't match "
+        "your environment"
+    ) in result.stdout
     assert len(result.files_created) == 0
 
 
@@ -647,16 +701,18 @@ def test_install_unsupported_wheel_file(script, data):
     # Trying to install a local wheel with an incompatible version/type
     # should fail.
     path = data.packages.joinpath("simple.dist-0.1-py1-none-invalid.whl")
-    script.scratch_path.joinpath("wheel-file.txt").write_text(textwrap.dedent("""\
-        {path}
-        """.format(**locals())))
+    script.scratch_path.joinpath("wheel-file.txt").write_text(path + "\n")
     result = script.pip(
-        'install', '-r', script.scratch_path / 'wheel-file.txt',
+        "install",
+        "-r",
+        script.scratch_path / "wheel-file.txt",
         expect_error=True,
         expect_stderr=True,
     )
-    assert ("simple.dist-0.1-py1-none-invalid.whl is not a supported " +
-            "wheel on this platform" in result.stderr)
+    assert (
+        "simple.dist-0.1-py1-none-invalid.whl is not a supported wheel on this platform"
+        in result.stderr
+    )
     assert len(result.files_created) == 0
 
 
@@ -682,17 +738,20 @@ def test_install_options_local_to_package(script, arg_recording_sdist_maker):
         )
     )
     script.pip(
-        'install',
-        '--no-index', '-f', str(simple1_sdist.sdist_path.parent),
-        '-r', reqs_file,
+        "install",
+        "--no-index",
+        "-f",
+        str(simple1_sdist.sdist_path.parent),
+        "-r",
+        reqs_file,
     )
 
     simple1_args = simple1_sdist.args()
-    assert 'install' in simple1_args
-    assert '-O0' in simple1_args
+    assert "install" in simple1_args
+    assert "-O0" in simple1_args
     simple2_args = simple2_sdist.args()
-    assert 'install' in simple2_args
-    assert '-O0' not in simple2_args
+    assert "install" in simple2_args
+    assert "-O0" not in simple2_args
 
 
 def test_location_related_install_option_fails(script):
@@ -700,9 +759,12 @@ def test_location_related_install_option_fails(script):
     reqs_file = script.scratch_path.joinpath("reqs.txt")
     reqs_file.write_text("simple --install-option='--home=/tmp'")
     result = script.pip(
-        'install',
-        '--no-index', '-f', str(simple_sdist.parent),
-        '-r', reqs_file,
-        expect_error=True
+        "install",
+        "--no-index",
+        "-f",
+        str(simple_sdist.parent),
+        "-r",
+        reqs_file,
+        expect_error=True,
     )
     assert "['--home'] from simple" in result.stderr
