@@ -7,7 +7,8 @@ from pip._internal.utils.urls import path_to_url
 from tests.lib import create_basic_sdist_for_package, create_basic_wheel_for_package
 
 _FindLinks = collections.namedtuple(
-    "_FindLinks", "index_html sdist_hash wheel_hash",
+    "_FindLinks",
+    "index_html sdist_hash wheel_hash",
 )
 
 
@@ -75,9 +76,11 @@ def test_new_resolver_hash_intersect(script, requirements_template, message):
         "--no-cache-dir",
         "--no-deps",
         "--no-index",
-        "--find-links", find_links.index_html,
-        "--verbose",
-        "--requirement", requirements_txt,
+        "--find-links",
+        find_links.index_html,
+        "-vv",
+        "--requirement",
+        requirements_txt,
     )
 
     assert message.format(name="base") in result.stdout, str(result)
@@ -107,10 +110,13 @@ def test_new_resolver_hash_intersect_from_constraint(script):
         "--no-cache-dir",
         "--no-deps",
         "--no-index",
-        "--find-links", find_links.index_html,
-        "--verbose",
-        "--constraint", constraints_txt,
-        "--requirement", requirements_txt,
+        "--find-links",
+        find_links.index_html,
+        "-vv",
+        "--constraint",
+        constraints_txt,
+        "--requirement",
+        requirements_txt,
     )
 
     message = (
@@ -138,7 +144,9 @@ def test_new_resolver_hash_intersect_from_constraint(script):
     ids=["both-requirements", "one-each"],
 )
 def test_new_resolver_hash_intersect_empty(
-    script, requirements_template, constraints_template,
+    script,
+    requirements_template,
+    constraints_template,
 ):
     find_links = _create_find_links(script)
 
@@ -163,9 +171,12 @@ def test_new_resolver_hash_intersect_empty(
         "--no-cache-dir",
         "--no-deps",
         "--no-index",
-        "--find-links", find_links.index_html,
-        "--constraint", constraints_txt,
-        "--requirement", requirements_txt,
+        "--find-links",
+        find_links.index_html,
+        "--constraint",
+        constraints_txt,
+        "--requirement",
+        requirements_txt,
         expect_error=True,
     )
 
@@ -193,8 +204,10 @@ def test_new_resolver_hash_intersect_empty_from_constraint(script):
         "--no-cache-dir",
         "--no-deps",
         "--no-index",
-        "--find-links", find_links.index_html,
-        "--constraint", constraints_txt,
+        "--find-links",
+        find_links.index_html,
+        "--constraint",
+        constraints_txt,
         "base==0.1.0",
         expect_error=True,
     )
@@ -204,3 +217,87 @@ def test_new_resolver_hash_intersect_empty_from_constraint(script):
         "from some requirements."
     )
     assert message in result.stderr, str(result)
+
+
+@pytest.mark.parametrize("constrain_by_hash", [False, True])
+def test_new_resolver_hash_requirement_and_url_constraint_can_succeed(
+    script,
+    constrain_by_hash,
+):
+    wheel_path = create_basic_wheel_for_package(script, "base", "0.1.0")
+
+    wheel_hash = hashlib.sha256(wheel_path.read_bytes()).hexdigest()
+
+    requirements_txt = script.scratch_path / "requirements.txt"
+    requirements_txt.write_text(
+        """
+        base==0.1.0 --hash=sha256:{wheel_hash}
+        """.format(
+            wheel_hash=wheel_hash,
+        ),
+    )
+
+    constraints_txt = script.scratch_path / "constraints.txt"
+    constraint_text = "base @ {wheel_url}\n".format(wheel_url=path_to_url(wheel_path))
+    if constrain_by_hash:
+        constraint_text += "base==0.1.0 --hash=sha256:{wheel_hash}\n".format(
+            wheel_hash=wheel_hash,
+        )
+    constraints_txt.write_text(constraint_text)
+
+    script.pip(
+        "install",
+        "--no-cache-dir",
+        "--no-index",
+        "--constraint",
+        constraints_txt,
+        "--requirement",
+        requirements_txt,
+    )
+
+    script.assert_installed(base="0.1.0")
+
+
+@pytest.mark.parametrize("constrain_by_hash", [False, True])
+def test_new_resolver_hash_requirement_and_url_constraint_can_fail(
+    script,
+    constrain_by_hash,
+):
+    wheel_path = create_basic_wheel_for_package(script, "base", "0.1.0")
+    other_path = create_basic_wheel_for_package(script, "other", "0.1.0")
+
+    other_hash = hashlib.sha256(other_path.read_bytes()).hexdigest()
+
+    requirements_txt = script.scratch_path / "requirements.txt"
+    requirements_txt.write_text(
+        """
+        base==0.1.0 --hash=sha256:{other_hash}
+        """.format(
+            other_hash=other_hash,
+        ),
+    )
+
+    constraints_txt = script.scratch_path / "constraints.txt"
+    constraint_text = "base @ {wheel_url}\n".format(wheel_url=path_to_url(wheel_path))
+    if constrain_by_hash:
+        constraint_text += "base==0.1.0 --hash=sha256:{other_hash}\n".format(
+            other_hash=other_hash,
+        )
+    constraints_txt.write_text(constraint_text)
+
+    result = script.pip(
+        "install",
+        "--no-cache-dir",
+        "--no-index",
+        "--constraint",
+        constraints_txt,
+        "--requirement",
+        requirements_txt,
+        expect_error=True,
+    )
+
+    assert (
+        "THESE PACKAGES DO NOT MATCH THE HASHES FROM THE REQUIREMENTS FILE."
+    ) in result.stderr, str(result)
+
+    script.assert_not_installed("base", "other")
