@@ -7,7 +7,6 @@ from tests.lib.wheel import make_wheel
 
 @pytest.fixture()
 def make_fake_wheel(script):
-
     def _make_fake_wheel(wheel_tag):
         wheel_house = script.scratch_path.joinpath("wheelhouse")
         wheel_house.mkdir()
@@ -39,8 +38,10 @@ def test_new_resolver_target_checks_compatibility_failure(
     args = [
         "install",
         "--only-binary=:all:",
-        "--no-cache-dir", "--no-index",
-        "--target", str(script.scratch_path.joinpath("target")),
+        "--no-cache-dir",
+        "--no-index",
+        "--target",
+        str(script.scratch_path.joinpath("target")),
         make_fake_wheel(fake_wheel_tag),
     ]
     if implementation:
@@ -58,7 +59,7 @@ def test_new_resolver_target_checks_compatibility_failure(
         abi,
         platform,
     )
-    wheel_tag_matches = (args_tag == fake_wheel_tag)
+    wheel_tag_matches = args_tag == fake_wheel_tag
 
     result = script.pip(*args, expect_error=(not wheel_tag_matches))
 
