@@ -1,8 +1,8 @@
 import logging
 from threading import Thread
+from unittest.mock import patch
 
 import pytest
-from mock import patch
 
 from pip._internal.utils.logging import (
     BrokenStdoutLoggingError,
@@ -18,7 +18,7 @@ logger = logging.getLogger(__name__)
 class TestIndentingFormatter:
     """Test ``pip._internal.utils.logging.IndentingFormatter``."""
 
-    def make_record(self, msg, level_name):
+    def make_record(self, msg: str, level_name: str) -> logging.LogRecord:
         level_number = getattr(logging, level_name)
         attrs = dict(
             msg=msg,
@@ -31,59 +31,72 @@ class TestIndentingFormatter:
 
         return record
 
-    @pytest.mark.parametrize('level_name, expected', [
-        ('DEBUG', 'hello\nworld'),
-        ('INFO', 'hello\nworld'),
-        ('WARNING', 'WARNING: hello\nworld'),
-        ('ERROR', 'ERROR: hello\nworld'),
-        ('CRITICAL', 'ERROR: hello\nworld'),
-    ])
-    def test_format(self, level_name, expected, utc):
+    @pytest.mark.parametrize(
+        "level_name, expected",
+        [
+            ("DEBUG", "hello\nworld"),
+            ("INFO", "hello\nworld"),
+            ("WARNING", "WARNING: hello\nworld"),
+            ("ERROR", "ERROR: hello\nworld"),
+            ("CRITICAL", "ERROR: hello\nworld"),
+        ],
+    )
+    def test_format(self, level_name: str, expected: str, utc: None) -> None:
         """
         Args:
           level_name: a logging level name (e.g. "WARNING").
         """
-        record = self.make_record('hello\nworld', level_name=level_name)
+        record = self.make_record("hello\nworld", level_name=level_name)
         f = IndentingFormatter(fmt="%(message)s")
         assert f.format(record) == expected
 
-    @pytest.mark.parametrize('level_name, expected', [
-        ('INFO',
-         '2019-01-17T06:00:37,040 hello\n'
-         '2019-01-17T06:00:37,040 world'),
-        ('WARNING',
-         '2019-01-17T06:00:37,040 WARNING: hello\n'
-         '2019-01-17T06:00:37,040 world'),
-    ])
-    def test_format_with_timestamp(self, level_name, expected, utc):
-        record = self.make_record('hello\nworld', level_name=level_name)
+    @pytest.mark.parametrize(
+        "level_name, expected",
+        [
+            ("INFO", "2019-01-17T06:00:37,040 hello\n2019-01-17T06:00:37,040 world"),
+            (
+                "WARNING",
+                "2019-01-17T06:00:37,040 WARNING: hello\n"
+                "2019-01-17T06:00:37,040 world",
+            ),
+        ],
+    )
+    def test_format_with_timestamp(
+        self, level_name: str, expected: str, utc: None
+    ) -> None:
+        record = self.make_record("hello\nworld", level_name=level_name)
         f = IndentingFormatter(fmt="%(message)s", add_timestamp=True)
         assert f.format(record) == expected
 
-    @pytest.mark.parametrize('level_name, expected', [
-        ('WARNING', 'DEPRECATION: hello\nworld'),
-        ('ERROR', 'DEPRECATION: hello\nworld'),
-        ('CRITICAL', 'DEPRECATION: hello\nworld'),
-    ])
-    def test_format_deprecated(self, level_name, expected, utc):
+    @pytest.mark.parametrize(
+        "level_name, expected",
+        [
+            ("WARNING", "DEPRECATION: hello\nworld"),
+            ("ERROR", "DEPRECATION: hello\nworld"),
+            ("CRITICAL", "DEPRECATION: hello\nworld"),
+        ],
+    )
+    def test_format_deprecated(self, level_name: str, expected: str, utc: None) -> None:
         """
         Test that logged deprecation warnings coming from deprecated()
         don't get another prefix.
         """
         record = self.make_record(
-            'DEPRECATION: hello\nworld', level_name=level_name,
+            "DEPRECATION: hello\nworld",
+            level_name=level_name,
         )
         f = IndentingFormatter(fmt="%(message)s")
         assert f.format(record) == expected
 
-    def test_thread_safety_base(self, utc):
+    def test_thread_safety_base(self, utc: None) -> None:
         record = self.make_record(
-            'DEPRECATION: hello\nworld', level_name='WARNING',
+            "DEPRECATION: hello\nworld",
+            level_name="WARNING",
         )
         f = IndentingFormatter(fmt="%(message)s")
         results = []
 
-        def thread_function():
+        def thread_function() -> None:
             results.append(f.format(record))
 
         thread_function()
@@ -92,14 +105,15 @@ class TestIndentingFormatter:
         thread.join()
         assert results[0] == results[1]
 
-    def test_thread_safety_indent_log(self, utc):
+    def test_thread_safety_indent_log(self, utc: None) -> None:
         record = self.make_record(
-            'DEPRECATION: hello\nworld', level_name='WARNING',
+            "DEPRECATION: hello\nworld",
+            level_name="WARNING",
         )
         f = IndentingFormatter(fmt="%(message)s")
         results = []
 
-        def thread_function():
+        def thread_function() -> None:
             with indent_log():
                 results.append(f.format(record))
 
@@ -111,16 +125,15 @@ class TestIndentingFormatter:
 
 
 class TestColorizedStreamHandler:
-
-    def _make_log_record(self):
+    def _make_log_record(self) -> logging.LogRecord:
         attrs = {
-            'msg': 'my error',
+            "msg": "my error",
         }
         record = logging.makeLogRecord(attrs)
 
         return record
 
-    def test_broken_pipe_in_stderr_flush(self):
+    def test_broken_pipe_in_stderr_flush(self) -> None:
         """
         Test sys.stderr.flush() raising BrokenPipeError.
 
@@ -130,20 +143,20 @@ class TestColorizedStreamHandler:
 
         with captured_stderr() as stderr:
             handler = ColorizedStreamHandler(stream=stderr)
-            with patch('sys.stderr.flush') as mock_flush:
+            with patch("sys.stderr.flush") as mock_flush:
                 mock_flush.side_effect = BrokenPipeError()
                 # The emit() call raises no exception.
                 handler.emit(record)
 
             err_text = stderr.getvalue()
 
-        assert err_text.startswith('my error')
+        assert err_text.startswith("my error")
         # Check that the logging framework tried to log the exception.
-        assert 'Logging error' in err_text
-        assert 'BrokenPipeError' in err_text
+        assert "Logging error" in err_text
+        assert "BrokenPipeError" in err_text
         assert "Message: 'my error'" in err_text
 
-    def test_broken_pipe_in_stdout_write(self):
+    def test_broken_pipe_in_stdout_write(self) -> None:
         """
         Test sys.stdout.write() raising BrokenPipeError.
 
@@ -153,12 +166,12 @@ class TestColorizedStreamHandler:
 
         with captured_stdout() as stdout:
             handler = ColorizedStreamHandler(stream=stdout)
-            with patch('sys.stdout.write') as mock_write:
+            with patch("sys.stdout.write") as mock_write:
                 mock_write.side_effect = BrokenPipeError()
                 with pytest.raises(BrokenStdoutLoggingError):
                     handler.emit(record)
 
-    def test_broken_pipe_in_stdout_flush(self):
+    def test_broken_pipe_in_stdout_flush(self) -> None:
         """
         Test sys.stdout.flush() raising BrokenPipeError.
 
@@ -168,7 +181,7 @@ class TestColorizedStreamHandler:
 
         with captured_stdout() as stdout:
             handler = ColorizedStreamHandler(stream=stdout)
-            with patch('sys.stdout.flush') as mock_flush:
+            with patch("sys.stdout.flush") as mock_flush:
                 mock_flush.side_effect = BrokenPipeError()
                 with pytest.raises(BrokenStdoutLoggingError):
                     handler.emit(record)
@@ -177,4 +190,4 @@ class TestColorizedStreamHandler:
 
         # Sanity check that the log record was written, since flush() happens
         # after write().
-        assert output.startswith('my error')
+        assert output.startswith("my error")
